@@ -10,19 +10,38 @@ class PyXploraApi:
         self._userLang = userLang
         self._timeZone = timeZone
         self._watchNo = watchNo
+        self.__handler: GQLHandler = []
+        self.__login()
+
+    def __checkLogin(self) -> None:
+        self.__handler = GQLHandler(self._countryPhoneNumber, self._phoneNumber, self._password, self._userLang, self._timeZone)
+        self.__handler.login()
+        #print("Login!")
         self.update()
 
-    def update(self):
-        self.handler = GQLHandler(self._countryPhoneNumber, self._phoneNumber, self._password, self._userLang, self._timeZone)
-        self.handler.login()
+    def __login(self) -> None:
+        if not self.__handler:
+            try:
+                self.__checkLogin()
+            except LoginError as error:
+                #print(f"Error: -> First faill. {error.args[0]}")
+                try:
+                    self.__checkLogin()
+                except LoginError as error:
+                    #print(f"Error: -> Login canceled! {error.args[0]}")
+                    raise Exception(f"{error.args[0]}")
+
+    def update(self) -> None:
+        self.__login()
+        #print("update")
 
         self.watch_no = self._watchNo
 
-        self.myInfo = self.handler.getMyInfo()['readMyInfo']
+        self.myInfo = self.__handler.getMyInfo()['readMyInfo']
         self.watch_user_id = self.myInfo['children'][self.watch_no]['ward']['id']
         self.watch_user_name = self.myInfo['children'][self.watch_no]['ward']['name']
 
-        self.watch_last_location = self.handler.getWatchLastLocation(self.watch_user_id)['watchLastLocate']
+        self.watch_last_location = self.__handler.getWatchLastLocation(self.watch_user_id)['watchLastLocate']
 
         self.contacts = []
         self.alarms = []
@@ -30,9 +49,13 @@ class PyXploraApi:
         self.safe_zones = []
         self.school_silent_mode = []
 
+    def version(self) -> str:
+        return "1.0.27"
+
 ##### Contact Info #####
     def getContacts(self) -> list:
-        for contact in self.handler.getContacts(self.watch_user_id)['contacts']['contacts']:
+        self.__login()
+        for contact in self.__handler.getContacts(self.watch_user_id)['contacts']['contacts']:
             try:
                 xcoin = contact['contactUser']['xcoin']
                 id = contact['contactUser']['id']
@@ -52,35 +75,41 @@ class PyXploraApi:
 
 ##### User Info #####
     def getUserID(self) -> str:
-        return self.myInfo['id']
+        return self.__checkMyInfo(self.myInfo, 'id', 'n/a')
     def getUserName(self) -> str:
-        return self.myInfo['name']
+        return self.__checkMyInfo(self.myInfo, 'name', 'n/a')
     def getUserIcon(self) -> str:
-        return self.myInfo['extra']['profileIcon']
+        return self.__checkMyInfo(self.__checkMyInfo(self.myInfo, 'extra', 'n/a'), 'profileIcon', 'n/a')
     def getUserXcoin(self) -> int:
-        return self.myInfo['xcoin']
+        return self.__checkMyInfo(self.myInfo, 'xcoin', -1)
     def getUserCurrentStep(self) -> int:
-        return self.myInfo['currentStep']
+        return self.__checkMyInfo(self.myInfo, 'currentStep', -1)
     def getUserTotalStep(self) -> int:
-        return self.myInfo['totalStep']
+        return self.__checkMyInfo(self.myInfo, 'totalStep', -1)
     def getUserCreate(self) -> str:
-        return datetime.fromtimestamp(self.myInfo['create']).strftime('%Y-%m-%d %H:%M:%S')
+        return datetime.fromtimestamp(self.__checkMyInfo(self.myInfo, 'create', 0)).strftime('%Y-%m-%d %H:%M:%S')
     def getUserUpdate(self) -> str:
-        return datetime.fromtimestamp(self.myInfo['update']).strftime('%Y-%m-%d %H:%M:%S')
+        return datetime.fromtimestamp(self.__checkMyInfo(self.myInfo, 'update', 0)).strftime('%Y-%m-%d %H:%M:%S')
 
 ##### Watch Info #####
     def getWatchUserID(self) -> str:
+        self.__login()
         return self.watch_user_id
     def getWatchUserName(self) -> str:
+        self.__login()
         return self.watch_user_name
     def getWatchXcoin(self) -> int:
+        self.__login()
         return self.myInfo['children'][self.watch_no]['ward']['xcoin']
     def getWatchCurrentStep(self) -> int:
+        self.__login()
         return self.myInfo['children'][self.watch_no]['ward']['currentStep']
     def getWatchTotalStep(self) -> int:
+        self.__login()
         return self.myInfo['children'][self.watch_no]['ward']['totalStep']
     def getWatchAlarm(self) -> list:
-        for alarm in self.handler.getAlarms(self.watch_user_id)['alarms']:
+        self.__login()
+        for alarm in self.__handler.getAlarms(self.watch_user_id)['alarms']:
             self.alarms.append({
                 'name': alarm['name'],
                 'start': alarm['start'],
@@ -89,41 +118,42 @@ class PyXploraApi:
             })
         return self.alarms
     def getWatchBattery(self) -> int:
+        self.__login()
         return self.watch_last_location['battery']
-    async def getWatchBattery_a(self) -> int:
-        await self.askWatchLocate_async()
+    async def __getWatchBattery_a(self) -> int:
+        await self.__askWatchLocate_async()
         await asyncio.sleep(15)
-        self.watch_last_location = (await self.handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
+        self.watch_last_location = (await self.__handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
         if self.watch_last_location != None:
             if (int(datetime.timestamp(datetime.now())) - self.watch_last_location['tm']) < 30:
                 return self.watch_last_location['battery']
-        watch_location = await self.askHelper()
+        watch_location = await self.__askHelper()
         await asyncio.sleep(1)
         return watch_location['battery']
-
     def getWatchIsCharging(self) -> bool:
-        return self.watch_last_location['isCharging']
-    async def getWatchIsCharging_a(self) -> bool:
-        await self.askWatchLocate_async()
-        await asyncio.sleep(15)
-        self.watch_last_location = (await self.handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
-        if self.watch_last_location != None:
-            if (int(datetime.timestamp(datetime.now())) - self.watch_last_location['tm']) < 30:
-                return self.watch_last_location['isCharging']
-        watch_location = await self.askHelper()
-        await asyncio.sleep(1)
-        return watch_location['isCharging']
+        self.__login()
+        try:
+            return self.watch_last_location['isCharging']
+        except TypeError:
+            return False
     def getWatchOnlineStatus(self) -> WatchOnlineStatus:
+        if self.askWatchLocate() == True:
+            return WatchOnlineStatus.ONLINE.value
+        self.__handler: GQLHandler = []
+        if self.askWatchLocate() == True:
+            return WatchOnlineStatus.ONLINE.value
         try:
             if self.trackWatchInterval() == -1:
                 return WatchOnlineStatus.OFFLINE.value
-            return self.handler.getWatches(self.watch_user_id)['watches'][self.watch_no]['onlineStatus']
+            return self.__handler.getWatches(self.watch_user_id)['watches'][self.watch_no]['onlineStatus']
         except TypeError:
             return WatchOnlineStatus.UNKNOWN.value
     def getWatchUnReadChatMsgCount(self) -> int: # bug?
-        return self.handler.unReadChatMsgCount(self.watch_user_id)['unReadChatMsgCount']
+        self.__login()
+        return self.__handler.unReadChatMsgCount(self.watch_user_id)['unReadChatMsgCount']
     def getWatchChats(self) -> list: # bug?
-        for chat in self.handler.chats(self.watch_user_id)['chats']['list']:
+        self.__login()
+        for chat in self.__handler.chats(self.watch_user_id)['chats']['list']:
             self.chats.append({
                 'msgId': chat['msgId'],
                 'type': chat['type'],
@@ -141,41 +171,32 @@ class PyXploraApi:
 
 ##### Watch Location Info #####
     def getWatchLastLocation(self) -> dict:
+        self.__login()
         return self.watch_last_location
     def getWatchLocateType(self) -> str:
+        self.__login()
         return self.watch_last_location['locateType']
-    async def getWatchLocate(self) -> dict:
-        await self.askWatchLocate_async()
-        await asyncio.sleep(15)
-        if (int(datetime.timestamp(datetime.now())) - self.watch_last_location['tm']) < 30:
-            return {
-                'tm': datetime.fromtimestamp(self.watch_last_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
-                'lat': self.watch_last_location['lat'],
-                'lng': self.watch_last_location['lng'],
-                'rad': self.watch_last_location['rad'],
-                'poi': self.watch_last_location['poi'],
-                'city': self.watch_last_location['city'],
-                'province': self.watch_last_location['province'],
-                'country': self.watch_last_location['country'],
-            }
-        watch_location = (await self.askHelper())
-        await asyncio.sleep(1)
+    def getWatchLocate(self) -> dict:
+        self.__login()
         return {
-            'tm': datetime.fromtimestamp(watch_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
-            'lat': watch_location['lat'],
-            'lng': watch_location['lng'],
-            'rad': watch_location['rad'],
-            'poi': watch_location['poi'],
-            'city': watch_location['city'],
-            'province': watch_location['province'],
-            'country': watch_location['country'],
+            'tm': datetime.fromtimestamp(self.watch_last_location['tm']).strftime('%Y-%m-%d %H:%M:%S'),
+            'lat': self.watch_last_location['lat'],
+            'lng': self.watch_last_location['lng'],
+            'rad': self.watch_last_location['rad'],
+            'poi': self.watch_last_location['poi'],
+            'city': self.watch_last_location['city'],
+            'province': self.watch_last_location['province'],
+            'country': self.watch_last_location['country'],
         }
     def getWatchIsInSafeZone(self) -> bool:
+        self.__login()
         return self.watch_last_location['isInSafeZone']
     def getWatchSafeZoneLabel(self) -> str:
+        self.__login()
         return self.watch_last_location['safeZoneLabel']
     def getSafeZones(self) -> list:
-        for safeZone in self.handler.safeZones(self.watch_user_id)['safeZones']:
+        self.__login()
+        for safeZone in self.__handler.safeZones(self.watch_user_id)['safeZones']:
             self.safe_zones.append({
                 #safeZone,
                 'groupName': safeZone['groupName'],
@@ -187,51 +208,64 @@ class PyXploraApi:
             })
         return self.safe_zones
     def trackWatchInterval(self) -> int:
-        return self.handler.trackWatch(self.watch_user_id)['trackWatch']
+        self.__login()
+        return self.__handler.trackWatch(self.watch_user_id)['trackWatch']
     def askWatchLocate(self) -> bool:
-        return self.handler.askWatchLocate(self.watch_user_id)['askWatchLocate']
-    async def askWatchLocate_async(self) -> bool:
-        return (await self.handler.askWatchLocate_a(self.watch_user_id))['askWatchLocate']
+        self.__login()
+        return self.__handler.askWatchLocate(self.watch_user_id)['askWatchLocate']
+    async def __askWatchLocate_async(self) -> bool:
+        self.__login()
+        return (await self.__handler.askWatchLocate_a(self.watch_user_id))['askWatchLocate']
 
 ##### Feature #####
     def schoolSilentMode(self) -> list:
-        for sientTime in self.handler.silentTimes(self.watch_user_id)['silentTimes']:
+        self.__login()
+        for sientTime in self.__handler.silentTimes(self.watch_user_id)['silentTimes']:
             self.school_silent_mode.append({
                 'id': sientTime['id'],
-                'start': self.helperTime(sientTime['start']),
-                'end': self.helperTime(sientTime['end']),
+                'start': self.__helperTime(sientTime['start']),
+                'end': self.__helperTime(sientTime['end']),
                 'weekRepeat': sientTime['weekRepeat'],
                 'status': sientTime['status'],
             })
         return self.school_silent_mode
     def setEnableSilentTime(self, silentId) -> bool:
-        return bool(self.handler.setEnableSlientTime(silentId)['setEnableSilentTime'])
+        self.__login()
+        return bool(self.__handler.setEnableSlientTime(silentId)['setEnableSilentTime'])
     def setDisableSilentTime(self, silentId) -> bool:
-        return bool(self.handler.setEnableSlientTime(silentId, NormalStatus.DISABLE)['setEnableSilentTime'])
+        self.__login()
+        return bool(self.__handler.setEnableSlientTime(silentId, NormalStatus.DISABLE)['setEnableSilentTime'])
     def setAllEnableSilentTime(self) -> list:
         res = []
         for silentTime in self.schoolSilentMode():
             res.append(bool(self.setEnableSilentTime(silentTime['id'])))
         return res
-    def setAllDisableSilentTime(self):
+    def setAllDisableSilentTime(self) -> list:
         res = []
         for silentTime in self.schoolSilentMode():
             res.append(bool(self.setDisableSilentTime(silentTime['id'])))
         return res
     def sendText(self, text): # sender is login User
-        return self.handler.sendText(self.watch_user_id, text)
+        self.__login()
+        return self.__handler.sendText(self.watch_user_id, text)
     def shutdown(self) -> bool:
-        return self.handler.shutdown(self.watch_user_id)
+        self.__login()
+        return self.__handler.shutdown(self.watch_user_id)
     def reboot(self) -> bool:
-        return self.handler.reboot(self.watch_user_id)
+        self.__login()
+        return self.__handler.reboot(self.watch_user_id)
 
 ##### - #####
-    def helperTime(self, time) -> str:
+    def __helperTime(self, time) -> str:
         h = str(int(time) /60).split('.')
         h2 = str(int(h[1]) *60).zfill(2)[:2]
         return h[0].zfill(2) + ":" + str(h2).zfill(2)
-    async def askHelper(self) -> int:
+    async def __askHelper(self) -> int:
         try:
-            return (await self.handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
+            return (await self.__handler.getWatchLastLocation_a(self.watch_user_id))['watchLastLocate']
         except TypeError:
-            return 0
\ No newline at end of file
+            return 0
+    def __checkMyInfo(self, func, value: str, res):
+        if value in func:
+            return func[value]
+        return res
